@@ -62,3 +62,12 @@ class ConcurrencyException(TrytonException):
 
 class RateLimitException(TrytonException):
     """User has sent too many requests in a given amount of time."""
+
+
+class MissingDependenciesException(TrytonException):
+
+    def __init__(self, missings):
+        self.missings = missings
+
+    def __unicode__(self):
+        return 'Missing dependencies: %s' % ' '.join(self.missings)
