@@ -52,7 +52,7 @@ class ModelStorage(Model):
     create_date = fields.Timestamp('Create Date', readonly=True)
     write_uid = fields.Many2One('res.user', 'Write User', readonly=True)
     write_date = fields.Timestamp('Write Date', readonly=True)
-    rec_name = fields.Function(fields.Char('Name'), 'get_rec_name',
+    rec_name = fields.Function(fields.Char('Record Name'), 'get_rec_name',
             searcher='search_rec_name')
 
     @classmethod
@@ -66,12 +66,12 @@ class ModelStorage(Model):
                     'write': RPC(readonly=False,
                         instantiate=slice(0, None, 2)),
                     'delete': RPC(readonly=False, instantiate=0),
-                    'copy': RPC(readonly=False, instantiate=0,
+                    'copy': RPC(readonly=False, instantiate=0, unique=False,
                         result=lambda r: map(int, r)),
                     'search': RPC(result=lambda r: map(int, r)),
                     'search_count': RPC(),
                     'search_read': RPC(),
-                    'export_data': RPC(instantiate=0),
+                    'export_data': RPC(instantiate=0, unique=False),
                     'import_data': RPC(readonly=False),
                     })
         cls._constraints = []
@@ -270,10 +270,16 @@ class ModelStorage(Model):
             if 'state' in cls._defaults:
                 default['state'] = cls._defaults['state']()
 
+        def is_readonly(Model):
+            return (not issubclass(Model, ModelStorage)
+                or (hasattr(Model, 'table_query')
+                    and Model.table_query()))
+
         def convert_data(field_defs, data):
             data = data.copy()
             for field_name in field_defs:
                 ftype = field_defs[field_name]['type']
+                field = cls._fields[field_name]
 
                 if field_name in (
                         'create_date',
@@ -285,9 +291,8 @@ class ModelStorage(Model):
 
                 if field_name in default:
                     data[field_name] = default[field_name]
-                elif (isinstance(cls._fields[field_name], fields.Function)
-                        and not isinstance(
-                            cls._fields[field_name], fields.MultiValue)):
+                if (isinstance(field, fields.Function)
+                        and not isinstance(field, fields.MultiValue)):
                     del data[field_name]
                 elif ftype in ('many2one', 'one2one'):
                     try:
@@ -296,10 +301,14 @@ class ModelStorage(Model):
                     except Exception:
                         pass
                 elif ftype in ('one2many',):
-                    if data[field_name]:
+                    if is_readonly(field.get_target()):
+                        del data[field_name]
+                    elif data[field_name]:
                         data[field_name] = [('copy', data[field_name])]
                 elif ftype == 'many2many':
-                    if data[field_name]:
+                    if is_readonly(pool.get(field.relation_name)):
+                        del data[field_name]
+                    elif data[field_name]:
                         data[field_name] = [('add', data[field_name])]
             if 'id' in data:
                 del data['id']
@@ -991,12 +1000,16 @@ class ModelStorage(Model):
 
                 validate_domain(field)
 
-                def required_test(value, field_name):
+                def required_test(value, field_name, field):
                     if (isinstance(value, (type(None), type(False), list,
                                     tuple, basestring, dict))
                             and not value):
                         cls.raise_user_error('required_validation_record',
                             error_args=cls._get_error_args(field_name))
+                    if (field._type == 'reference'
+                            and not isinstance(value, ModelStorage)):
+                        cls.raise_user_error('required_validation_record',
+                            error_args=cls._get_error_args(field_name))
                 # validate states required
                 if field.states and 'required' in field.states:
                     if is_pyson(field.states['required']):
@@ -1012,16 +1025,17 @@ class ModelStorage(Model):
                             required = PYSONDecoder(env).decode(pyson_required)
                             if required:
                                 required_test(getattr(record, field_name),
-                                    field_name)
+                                    field_name, field)
                     else:
                         if field.states['required']:
                             for record in records:
                                 required_test(getattr(record, field_name),
-                                    field_name)
+                                    field_name, field)
                 # validate required
                 if field.required:
                     for record in records:
-                        required_test(getattr(record, field_name), field_name)
+                        required_test(
+                            getattr(record, field_name), field_name, field)
                 # validate size
                 if hasattr(field, 'size') and field.size is not None:
                     for record in records:
@@ -1424,7 +1438,7 @@ class ModelStorage(Model):
                 to_write = []
                 for target in targets:
                     if target.id is None or target.id < 0:
-                        if field._type == 'one2many':
+                        if field._type == 'one2many' and field.field:
                             # Don't store old target link
                             setattr(target, field.field, None)
                         to_create.append(target._save_values)
@@ -1457,9 +1471,10 @@ class ModelStorage(Model):
             save_values = {}
             to_create = []
             to_write = []
-            transaction = records[0]._transaction
-            user = records[0]._user
-            context = records[0]._context
+            first = next(iter(records))
+            transaction = first._transaction
+            user = first._user
+            context = first._context
             for record in records:
                 if (record._transaction != transaction
                         or user != record._user
